@@ -1,5 +1,5 @@
 /*==================================================================
- * SwamiUIObject.c - Swami sound font GTK tree object
+ * SwamiUIObject.c - Swami main user interface object
  *
  * Swami
  * Copyright (C) 1999-2003 Josh Green <jgreen@users.sourceforge.net>
@@ -31,6 +31,10 @@
 #include <gtk/gtk.h>
 #include <string.h>
 
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
+
 #include <instpatch.h>
 
 #include <libswami/SwamiSamplelib.h>
@@ -150,6 +154,13 @@ GList *gen_clipboard = NULL;	/* list of IPGen values */
 static gboolean block_spanwin_select = FALSE;
 
 
+/* characters not included at the end of paired stereo sample names */
+static char *sample_stereo_skipchars = "\t _-([{";
+
+/* escaped characters in exported sample names */
+static char *sample_escchars = "\t\\/|";
+
+
 /* Local Prototypes */
 
 static void swamiui_object_class_init (SwamiUIObjectClass *klass);
@@ -183,6 +194,9 @@ static void swamiui_cb_item_properties_okay (GtkWidget *btn,
 					     GtkWidget *dialog);
 static void swamiui_cb_load_sample_okay (GtkWidget *multisel);
 static void swamiui_cb_add_samples (GtkWidget *multisel);
+static void cb_sample_type_selection_done (GtkWidget *menu, gpointer data);
+static void swamiui_export_samples_ok (GtkWidget *btn, GtkWidget *filesel);
+static char *make_sample_filename (IPSample *sample, char *path, char *ext);
 
 guint
 swamiui_object_get_type (void)
@@ -1190,8 +1204,7 @@ swamiui_save_files (GList *items, gboolean saveas)
     }
 
   /* item(s) have been changed, pop user interactive dialog */
-  multi = swamiui_multilist_new (_("Save files"),
-	 "*** Please don't distribute any saved sound fonts yet ***", 2,
+  multi = swamiui_multilist_new (_("Save files"), _("Save files"), 2,
 				 _("Name"), _("File"));
   gtk_signal_connect (GTK_OBJECT (SWAMIUI_MULTILIST (multi)->ok_button),
 		      "clicked", cb_save_files_ok, multi);
@@ -1761,6 +1774,303 @@ swamiui_cb_add_samples (GtkWidget *multisel)
   g_list_free (files);
 }
 
+/**
+ * swamiui_export_samples:
+ * @items: List of sample items to export to files.
+ */
+void
+swamiui_export_samples (GList *items)
+{
+  GtkWidget *filesel;
+  GtkWidget *opmenu;
+  GtkWidget *menu;
+  GtkWidget *mitem;
+  GtkWidget *hbox;
+  GtkWidget *label;
+  GList *newlist;
+
+  filesel = gtk_file_selection_new (_("Choose directory"));
+
+  /* pack a box with file type option menu */
+
+  hbox = gtk_hbox_new (FALSE, 4);
+  gtk_box_pack_start (GTK_BOX (GTK_FILE_SELECTION (filesel)->main_vbox),
+		      hbox, FALSE, FALSE, 0);
+
+  label = gtk_label_new ("File type:");
+  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+
+  opmenu = gtk_option_menu_new ();
+  gtk_widget_show (opmenu);
+  gtk_box_pack_start (GTK_BOX (hbox), opmenu, FALSE, TRUE, 0);
+
+  gtk_object_set_data (GTK_OBJECT (filesel), "opmenu", opmenu);
+
+  label = gtk_label_new ("Note: Loop/tuning info saved only with"
+			 " libaudiofile and AIFF files");
+  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+
+  menu = gtk_menu_new ();
+
+  /* hard coded file types, woo hoo! */
+
+  mitem = gtk_menu_item_new_with_label ("AIFF");
+  gtk_object_set_data (GTK_OBJECT (mitem), "type",
+		       GUINT_TO_POINTER (SWAMI_SAMPLELIB_TYPE_AIFF));
+  gtk_widget_show (mitem);
+  gtk_menu_append (GTK_MENU (menu), mitem);
+
+  mitem = gtk_menu_item_new_with_label ("WAV");
+  gtk_object_set_data (GTK_OBJECT (mitem), "type",
+		       GUINT_TO_POINTER (SWAMI_SAMPLELIB_TYPE_WAVE));
+  gtk_widget_show (mitem);
+  gtk_menu_append (GTK_MENU (menu), mitem);
+
+  mitem = gtk_menu_item_new_with_label ("AU");
+  gtk_object_set_data (GTK_OBJECT (mitem), "type",
+		       GUINT_TO_POINTER (SWAMI_SAMPLELIB_TYPE_AU));
+  gtk_widget_show (mitem);
+  gtk_menu_append (GTK_MENU (menu), mitem);
+
+  gtk_option_menu_set_menu (GTK_OPTION_MENU (opmenu), menu);
+
+  /* ugggh, GTK 1.2 really sucked with the option menu */
+  gtk_signal_connect (GTK_OBJECT (menu), "selection-done",
+		      GTK_SIGNAL_FUNC (cb_sample_type_selection_done),
+		      filesel);
+
+  gtk_widget_show_all (hbox);
+
+  newlist = g_list_copy (items);
+  gtk_object_set_data_full (GTK_OBJECT (filesel), "items", newlist,
+			    (GtkDestroyNotify)g_list_free);
+
+  /* default file type */
+  gtk_object_set_data (GTK_OBJECT (filesel), "type",
+		       GUINT_TO_POINTER (SWAMI_SAMPLELIB_TYPE_AIFF));
+
+  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filesel)->ok_button),
+    "clicked", (GtkSignalFunc) swamiui_export_samples_ok, filesel);
+
+  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (filesel)->
+      cancel_button), "clicked", (GtkSignalFunc) gtk_widget_destroy,
+    GTK_OBJECT (filesel));
+
+  gtk_widget_show (filesel);
+}
+
+/* much hate and death to the fucking GTK 1.2 option menu */
+static void
+cb_sample_type_selection_done (GtkWidget *menu, gpointer data)
+{
+  GtkWidget *filesel = GTK_WIDGET (data);
+  GtkWidget *mitem;
+  int filetype;
+
+  mitem = gtk_menu_get_active (GTK_MENU (menu));
+
+  filetype = GPOINTER_TO_UINT
+    (gtk_object_get_data (GTK_OBJECT (mitem), "type"));
+
+  gtk_object_set_data (GTK_OBJECT (filesel), "type",
+		       GUINT_TO_POINTER (filetype));
+}
+
+/* callback when user clicks OK on sample export file (directory) selector */
+static void
+swamiui_export_samples_ok (GtkWidget *btn, GtkWidget *filesel)
+{
+  const gchar *filename;
+  IPSample *sample;
+  IPSampleData *left, *right;
+  SwamiSamplelibParams params;
+  SwamiSamplelib *samplelib;
+  SwamiSamplelibHandle *handle;
+  GHashTable *saved_links; /* prevent multiple saves of stereo pairs */
+  GList *items, *p;
+  struct stat buf;
+  char *path, *filepath;
+  char *ext;
+  int filetype;
+
+  samplelib =
+    SWAMI_SAMPLELIB (swami_get_object_by_type (G_OBJECT (swami_object),
+					       "SwamiSamplelib"));
+  g_return_if_fail (samplelib != NULL);
+
+  saved_links = g_hash_table_new (NULL, NULL);
+
+  filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (filesel));
+  path = g_dirname (filename);	/* !! allocated string */
+
+  items = gtk_object_get_data (GTK_OBJECT (filesel), "items");
+
+  filetype = GPOINTER_TO_UINT
+    (gtk_object_get_data (GTK_OBJECT (filesel), "type"));
+
+  ext = swami_samplelib_type_ext (filetype);
+
+  /* check for write access to the path */
+  if (stat (path, &buf) != 0 || !S_ISDIR (buf.st_mode)
+      || access (path, W_OK) != 0)
+    {
+      g_critical ("Selected path not a directory or no write access");
+      g_free (path);
+    }
+
+  for (p = items; p; p = p->next)
+    {
+      /* not a sample or already saved its stereo linked sample? - skip */
+      if (!INSTP_IS_SAMPLE (p->data)
+	  || g_hash_table_lookup (saved_links, p->data))
+	continue;
+
+      sample = INSTP_SAMPLE (p->data);
+
+      swami_samplelib_set_params_from_sample (sample, &params);
+      params.file_type = filetype;
+
+      /* make a unique filename for the sample */
+      filepath = make_sample_filename (sample, path, ext);
+
+      handle = swami_samplelib_open (samplelib, filepath, 'w', &params);
+      if (!handle)
+	{
+	  g_critical ("Failed to create sample file '%s'", filepath);
+	  g_free (filepath);
+	  continue;
+	}
+
+      if (!sample->linked)
+	{
+	  left = sample->sampledata;
+	  right = NULL;
+	}
+      else			/* stereo */
+	{
+	  if (sample->sampletype & IPSAMPLE_TYPE_RIGHT)
+	    {
+	      right = sample->sampledata;
+	      left = sample->linked->sampledata;
+	    }
+	  else
+	    {
+	      left = sample->sampledata;
+	      right = sample->linked->sampledata;
+	    }
+
+	  g_hash_table_insert (saved_links, sample->linked,
+			       GUINT_TO_POINTER (TRUE));
+	}
+
+      if (swami_samplelib_save_sampledata (handle, left, right) != SWAMI_OK)
+	{
+	  swami_samplelib_close (handle);
+	  g_critical ("Failed to save sample data to file '%s'", filename);
+	  continue;
+	}
+
+      swami_samplelib_close (handle);
+    }
+
+  g_free (path);
+  g_hash_table_destroy (saved_links);
+
+  gtk_widget_destroy (filesel);
+}
+
+static char *
+make_sample_filename (IPSample *sample, char *path, char *ext)
+{
+  struct stat buf;
+  char *filepath;
+  char *name;
+  char *s1, *s2;
+  char numstr[6];
+  int len, i;
+
+  if (sample->linked)		/* stereo? */
+    {
+      /* attempt to strip off any L/R postfixes by using identical portions
+	 of sample names */
+
+      s1 = sample->name;
+      s2 = sample->linked->name;
+      for (; *s1 && *s1 == *s2; s1++, s2++); /* find matching portions */
+
+      if (s1 > sample->name)	/* anything in common? */
+	{
+	  /* find last character not in skip chars */
+	  do
+	    {
+	      s1--;
+	      if (!strchr (sample_stereo_skipchars, *s1))
+		{
+		  s1++;
+		  break;
+		}
+	    }
+	  while (s1 > sample->name);
+	}
+
+      if (s1 > sample->name)
+	name = g_strndup (sample->name, s1 - sample->name);
+      else	     /* nothing in common? - use this sample's name */
+	name = g_strdup (sample->name);
+    }
+  else				/* mono */
+    name = g_strdup (sample->name);
+
+  if (strlen (name) == 0)
+    {
+      g_free (name);
+      name = g_strdup ("untitled");
+    }
+
+  /* escape unwanted characters */
+  for (s1 = name; *s1; s1++)
+    if (strchr (sample_escchars, *s1)) *s1 = '_';
+
+  /* construct complete path with 5 chars padding in case a unique number
+     needs to be appended */
+  filepath = g_strconcat (path, G_DIR_SEPARATOR_S, name, ".", ext,
+			  "     ", NULL);
+  len = strlen (filepath);
+  filepath[len - 5] = '\0'; /* kill padding for now */
+
+  /* already exists? - find a unique number to append */
+  if (stat (filepath, &buf) == 0)
+    {
+      filepath[len - 5] = ' ';	/* set back to space */
+
+      filepath[len - strlen (ext) - 6] = '_';
+      filepath[len - strlen (ext) - 1] = '.';
+      strcpy (&filepath[len - strlen (ext)], ext);
+
+      s1 = &filepath[len - strlen (ext) - 5];
+      for (i = 0; i < 10000; i++)
+	{
+	  sprintf (numstr, "%04u", i);
+	  strncpy (s1, numstr, 4);
+	  if (stat (filepath, &buf) != 0)
+	    break;
+	}
+
+      if (i == 10000)
+	{
+	  g_critical ("Failed to find unique sample file name for '%s'",
+		      name);
+	  g_free (filepath);
+	  g_free (name);
+	  return (NULL);
+	}
+    }
+
+  g_free (name);
+
+  return (filepath);
+}
+
 /**
  * swamiui_unset_gens:
  * @items: List of patch items
