@@ -43,7 +43,7 @@ typedef enum {
   RMU_Save,
   RMU_Save_As,
 
-  RMU_Export,
+  RMU_Export_Samples,
 
   RMU_Unset_Gens,
   RMU_Copy_Gens,
@@ -90,7 +90,7 @@ static void swamiui_cb_paste (GList *item_list);
 static void swamiui_cb_wtbl_load_patch (GList *item_list);
 static void swamiui_cb_new_item (GList *item_list, gpointer data);
 static void swamiui_cb_goto_zone_refitem (GList *item_list);
-static void swamiui_cb_load_sample (GList *item_list);
+static void swamiui_cb_load_samples (GList *item_list);
 static void swamiui_cb_copy_gens (GList *item_list);
 static void swamiui_cb_paste_gens (GList *item_list);
 
@@ -108,7 +108,7 @@ static RMU_Item rmu_items[] = {
   { 'M', N_("Properties"), "<control>R", NULL, swamiui_item_properties, NULL },
   { 'M', N_("Save"), NULL, NULL, swamiui_save_files, GINT_TO_POINTER (FALSE) },
   { 'M', N_("Save As"), NULL, NULL, swamiui_save_files, GINT_TO_POINTER(TRUE)},
-  { 'M', N_("Export"), NULL, NULL, /* uisam_export_sample */ NULL, NULL },
+  { 'M', N_("Export Samples"), NULL, NULL, swamiui_export_samples, NULL },
 
   { 'M', N_("Unset Generators"), NULL, NULL, swamiui_unset_gens, NULL },
   { 'S', N_("Copy Generators"), NULL, NULL, swamiui_cb_copy_gens, NULL },
@@ -129,7 +129,7 @@ static RMU_Item rmu_items[] = {
   { 'S', N_("New Instrument"), NULL, NULL,
     swamiui_cb_new_item, GINT_TO_POINTER (IPITEM_INST) },
   { 'S', N_("Load Samples"), NULL, NULL,
-    swamiui_cb_load_sample, NULL },
+    swamiui_cb_load_samples, NULL },
   { 'S', N_("Global Zone"), "<control>G", NULL,
     swamiui_cb_new_item, GINT_TO_POINTER (IPITEM_ZONE) },
   { 'S', N_("Wavetable Load"), "<control>L", NULL,
@@ -219,7 +219,7 @@ static RMUItemType rmu_sampleroot[] = {	/* Menu for sample root tree */
 };
 static RMUItemType rmu_sample[] = {	/* Menu for samples */
   RMU_Properties,
-  RMU_Export,
+  RMU_Export_Samples,
   RMU_Delete,
   RMU_Wtbl_Load,
 
@@ -616,9 +616,9 @@ swamiui_cb_goto_zone_refitem (GList *item_list)
   swamiui_goto_zone_refitem (zone, klass->active_treemenu->tree);
 }
 
-/* wrapper for swamiui_load_sample */
+/* wrapper for swamiui_load_samples */
 static void
-swamiui_cb_load_sample (GList *item_list)
+swamiui_cb_load_samples (GList *item_list)
 {
   swamiui_load_samples (INSTP_ITEM (item_list->data));
 }
