@@ -720,7 +720,7 @@ ipfile_end_chunk (IPFileContext *ctx)
 
       /* write chunk size without adding to chunk size counter,
 	 we use virtual chunk size to handle dummy sample block */
-      size = state->vsize;
+      size = GUINT32_TO_LE (state->vsize);
       if (ipfile_write (ctx, &size, 4, FALSE) != INSTP_OK)
 	return (INSTP_FAIL);
 
