@@ -927,19 +927,9 @@ sfloader_preset_foreach_voice (IPItem *item, IPSample *sample,
     gen_array[IPGEN_SAMPLE_MODES].uword = IPSAMPLE_LOOP;
 
   /* set generator parameters */
-  for (i = 0; i < IPGEN_SAMPLE_MODES; i++)
+  for (i = 0; i < IPGEN_COUNT; i++)
     fluid_voice_gen_set (voice, i, (float)(gen_array[i].sword));
 
-  /* a hack because FluidSynth can't handle "loop till release" mode */
-  i = gen_array[IPGEN_SAMPLE_MODES].uword;
-  if (i == IPSAMPLE_LOOP_UNROLL) i = IPSAMPLE_LOOP;
-  fluid_voice_gen_set (voice, IPGEN_SAMPLE_MODES, (float)i);
-
-  /* set remaining generator parameters */
-  for (i = IPGEN_SAMPLE_MODES + 1; i < IPGEN_COUNT; i++)
-    fluid_voice_gen_set (voice, i, (float)(gen_array[i].sword));
-
-
   while (mods)
     {
       /*
